#include <ncurses.h>
#include <utility>
#include <vector>
#include <chrono>
#include <random>
#include <iostream>
#include <cmath>

float gravity = 20;

class Particle;

std::vector <Particle> particles;

std::chrono::high_resolution_clock timer;

class Particle
{
public:
    std::pair <float, float> pos;
    std::pair <float, float> vel;
    std::pair <float, float> acc;
    char character = '*';
    int color = 1;

    bool isRocket = false;
    double lifetime = rand() % 90;

    Particle(std::pair<float, float> position, char c, int col)
    {
        pos = position;
        character = c;
        color = col;
    }

    void Explode(int &i)
    {
        int explosionColor = rand() % 6 + 1;

        for (int i = 0; i < 64; i++)
        {
            Particle fire(pos, 36, explosionColor);
            fire.isRocket = false;
            particles.push_back(fire);

            float angle = (float(rand())/float((RAND_MAX)) * 2  * M_PI);
            float velocity = (float(rand())/float((RAND_MAX)) * 64);
            particles[particles.size() - 1].vel = std::make_pair(velocity * cos(angle), 2 * velocity * sin(angle));
        }

        particles.erase(particles.begin() + i);
        i--;
    }

    void Update(double deltatime, int &i)
    {
        if(pos.first < 0 || pos.first > getmaxy(stdscr) || pos.second < 0 || pos.second > getmaxx(stdscr))
        {
            particles.erase(particles.begin() + i);
            i--;
        }

        if(vel.first < 0 && isRocket)
        {
            Explode(i);
        }

        if(isRocket) character = 94;

        if(!isRocket)
        {
            vel.first *= pow(100, deltatime);
            vel.second *= pow(100, deltatime);
            lifetime += deltatime * 100;

            if(lifetime < 0)
            {
                particles.erase(particles.begin() + i);
                i--;
            } 
            character = 48 + floor(lifetime / 10);
        }

        vel.first += gravity * deltatime;

        vel.first += acc.first * deltatime;
        vel.second += acc.second * deltatime;

        pos.first += vel.first * deltatime;
        pos.second += vel.second * deltatime;

        acc.first = 0;
        acc.second = 0;
    }
};

void SendRocket()
{
    Particle p(std::make_pair(getmaxy(stdscr), rand() % getmaxx(stdscr)), 94, 7);
    p.vel.first = getmaxy(stdscr) / 2;
    p.isRocket = true;
    particles.push_back(p);
}

int main()
{
    srand(time(NULL));

    initscr();
    noecho();
    cbreak();
    curs_set(0);
    if(has_colors) start_color();

    for (int i = 0; i < 8; i++)
    {
        init_pair(i, i, 0);
    }
    
    auto last = timer.now();

    double rocketTimer = 0;

    while (true)
    {
        std::chrono::duration<double> elapsed = last - timer.now();
        double dt = elapsed.count();
        last = timer.now();

        rocketTimer += dt;

        if(rocketTimer < -0.4)
        {
            SendRocket();
            rocketTimer = 0;
        }


        for (int i = 0; i < particles.size(); i++)
        {
            particles[i].Update(dt, i);
        }

        erase();
        for (int i = 0; i < particles.size(); i++)
        {
            attron(COLOR_PAIR(particles[i].color));
            mvaddch(particles[i].pos.first, particles[i].pos.second, particles[i].character);
            attroff(COLOR_PAIR(particles[i].color));
        }
        refresh();
    }

    endwin();
    return(0);
}